==> v0.13.11.tar.gz: git-next/crates/core/src/git/user_notification.rs <==
use std::fmt::Display;

//
use crate::{git::Commit, BranchName, ForgeAlias, RepoAlias};
use serde_json::json;

use super::graph::Log;

#[derive(Clone, Debug, Hash, PartialEq, Eq)]
pub enum UserNotification {
    CICheckFailed {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        commit: Commit,
        log: Log,
    },
    RepoConfigLoadFailure {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        reason: String,
    },
    WebhookRegistration {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        reason: String,
    },
    DevNotBasedOnMain {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        dev_branch: BranchName,
        main_branch: BranchName,
        dev_commit: Commit,
        main_commit: Commit,
        log: Log,
    },
}
impl UserNotification {
    #[must_use]
    pub fn as_json(&self, timestamp: time::OffsetDateTime) -> serde_json::Value {
        let timestamp = timestamp.unix_timestamp().to_string();
        match self {
            Self::CICheckFailed {
                forge_alias,
                repo_alias,
                commit,
                log,
            } => json!({
                "type": "cicheck.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "commit": {
                        "sha": commit.sha(),
                        "message": commit.message()
                    },
                    "log": **log
                }
            }),
            Self::RepoConfigLoadFailure {
                forge_alias,
                repo_alias,
                reason,
            } => json!({
                "type": "config.load.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "reason": reason,
                }
            }),
            Self::WebhookRegistration {
                forge_alias,
                repo_alias,
                reason,
            } => json!({
                "type": "webhook.registration.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "reason": reason,
                }
            }),
            Self::DevNotBasedOnMain {
                forge_alias,
                repo_alias,
                dev_branch,
                main_branch,
                dev_commit,
                main_commit,
                log,
            } => json!({
                "type": "branch.dev.not-on-main",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "branches": {
                        "dev": dev_branch,
                        "main": main_branch
                    },
                    "commits": {
                        "dev": {
                            "sha": dev_commit.sha(),
                            "message": dev_commit.message()
                        },
                        "main": {
                            "sha": main_commit.sha(),
                            "message": main_commit.message()
                        }
                    },
                    "log": **log
                }
            }),
        }
    }

    #[must_use]
    pub fn aliases(&self) -> (ForgeAlias, RepoAlias) {
        match self {
            Self::CICheckFailed {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::RepoConfigLoadFailure {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::WebhookRegistration {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::DevNotBasedOnMain {
                forge_alias,
                repo_alias,
                ..
            } => (forge_alias.clone(), repo_alias.clone()),
        }
    }
}
impl Display for UserNotification {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let message = match self {
            Self::CICheckFailed { commit, .. } => format!("CI Check Failed [{commit}]"),
            Self::RepoConfigLoadFailure { reason, .. } => {
                format!("Failed to load repo config: {reason}")
            }
            Self::WebhookRegistration { reason, .. } => {
                format!("Failed to register webhook: {reason}")
            }
            Self::DevNotBasedOnMain {
                dev_branch,
                main_branch,
                ..
            } => format!("{dev_branch} not based on {main_branch}"),
        };
        write!(f, "{message}")
    }
}

==> v0.14.1.tar.gz: git-next/crates/core/src/git/user_notification.rs <==
use std::fmt::Display;

//
use crate::{git::Commit, BranchName, ForgeAlias, RepoAlias};
use serde_json::json;

use super::graph::Log;

#[derive(Clone, Debug, Hash, PartialEq, Eq)]
pub enum UserNotification {
    CICheckFailed {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        commit: Commit,
        log: Log,
    },
    RepoConfigLoadFailure {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        reason: String,
    },
    WebhookRegistration {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        reason: String,
    },
    DevNotBasedOnMain {
        forge_alias: ForgeAlias,
        repo_alias: RepoAlias,
        dev_branch: BranchName,
        main_branch: BranchName,
        dev_commit: Commit,
        main_commit: Commit,
        log: Log,
    },
}
impl UserNotification {
    #[must_use]
    pub fn as_json(&self, timestamp: time::OffsetDateTime) -> serde_json::Value {
        let timestamp = timestamp.unix_timestamp();
        match self {
            Self::CICheckFailed {
                forge_alias,
                repo_alias,
                commit,
                log,
            } => json!({
                "type": "cicheck.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "commit": {
                        "sha": commit.sha(),
                        "message": commit.message()
                    },
                    "log": **log
                }
            }),
            Self::RepoConfigLoadFailure {
                forge_alias,
                repo_alias,
                reason,
            } => json!({
                "type": "config.load.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "reason": reason,
                }
            }),
            Self::WebhookRegistration {
                forge_alias,
                repo_alias,
                reason,
            } => json!({
                "type": "webhook.registration.failed",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "reason": reason,
                }
            }),
            Self::DevNotBasedOnMain {
                forge_alias,
                repo_alias,
                dev_branch,
                main_branch,
                dev_commit,
                main_commit,
                log,
            } => json!({
                "type": "branch.dev.not-on-main",
                "timestamp": timestamp,
                "data": {
                    "forge_alias": forge_alias,
                    "repo_alias": repo_alias,
                    "branches": {
                        "dev": dev_branch,
                        "main": main_branch
                    },
                    "commits": {
                        "dev": {
                            "sha": dev_commit.sha(),
                            "message": dev_commit.message()
                        },
                        "main": {
                            "sha": main_commit.sha(),
                            "message": main_commit.message()
                        }
                    },
                    "log": **log
                }
            }),
        }
    }

    #[must_use]
    pub fn aliases(&self) -> (ForgeAlias, RepoAlias) {
        match self {
            Self::CICheckFailed {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::RepoConfigLoadFailure {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::WebhookRegistration {
                forge_alias,
                repo_alias,
                ..
            }
            | Self::DevNotBasedOnMain {
                forge_alias,
                repo_alias,
                ..
            } => (forge_alias.clone(), repo_alias.clone()),
        }
    }
}
impl Display for UserNotification {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let message = match self {
            Self::CICheckFailed { commit, .. } => format!("CI Check Failed [{commit}]"),
            Self::RepoConfigLoadFailure { reason, .. } => {
                format!("Failed to load repo config: {reason}")
            }
            Self::WebhookRegistration { reason, .. } => {
                format!("Failed to register webhook: {reason}")
            }
            Self::DevNotBasedOnMain {
                dev_branch,
                main_branch,
                ..
            } => format!("{dev_branch} not based on {main_branch}"),
        };
        write!(f, "{message}")
    }
}
